from datetime import date
from hashlib import md5
import logging
import os
import time

from dateutil.relativedelta import relativedelta
from django.conf import settings
from django.contrib.staticfiles import finders
from django.db import models
from django.core.exceptions import ObjectDoesNotExist
from django.utils.translation import ugettext_lazy as _
from geopy.geocoders import Nominatim
from ldapdb.models.fields import CharField, DateField, FloatField, ImageField, IntegerField, ListField
import ldapdb.models
from pam import pam

from .util import ssha

logger = logging.getLogger(__name__)

nominatim = Nominatim()

class TeckidsLdapMixin(object):
    @classmethod
    def get_by_dn(cls, dn):
        rdn = dn.split(',')[0]
        rdn_parts = {rdn_part.split('=')[0]: rdn_part.split('=')[1] for rdn_part in rdn.split('+')}
        return cls.objects.get(**rdn_parts)

    def _get_objects(self, field, cls):
        objects = []
        
        for dn in getattr(self, field):
            if not dn.endswith(self.base_dn):
                try:
                    objects.append(cls.get_by_dn(dn))
                except ObjectDoesNotExist as exception:
                    # Do not break on broken member DNs
                    logger.error('Getting %s by DN %s failed: %s' % (cls.__name__, dn, str(exception)))
            else:
                objects += self.get_by_dn(dn).members_persons

        return objects

    def get_image_rel(self, field='jpeg_photo', fallback=None):
        entry_dir = os.path.join(settings.MEDIA_ROOT, 'ticdesk', 'ldap', md5(self.dn.encode("utf-8")).hexdigest())

        if not os.path.isdir(entry_dir):
            os.makedirs(entry_dir)

        filename = '%s.jpg' % os.path.join(entry_dir, md5(field.encode("utf-8")).hexdigest())

        if getattr(self, field):
            content = getattr(self, field)
        else:
            if fallback is None:
                fallback = finders.find('people.jpg')
            with open(fallback, 'rb') as fallback_file:
                content = fallback_file.read()

        with open(filename, 'wb') as image_file:
            image_file.write(content)

        return '%s/%s/%s/%s.jpg' % ('ticdesk', 'ldap', md5(self.dn.encode("utf-8")).hexdigest(), md5(field.encode("utf-8")).hexdigest())

    def get_image_url(self, field='jpeg_photo', fallback=None):
        return '%s%s' % (settings.MEDIA_URL, self.get_image_rel(field, fallback))

    def get_image_path(self, field='jpeg_photo', fallback=None):
        return '%s/%s' % (settings.MEDIA_ROOT, self.get_image_rel(field, fallback))

    def __getattr__(self, attr):
        if attr.endswith('__url'):
            return self.get_image_url(attr[:-5])
        elif attr.endswith('__path'):
            return self.get_image_path(attr[:-6])
        elif attr.endswith('__rel'):
            return self.get_image_rel(attr[:-5])

class TeckidsGroup(ldapdb.models.Model, TeckidsLdapMixin):
    class Meta:
        managed = False
        app_label = 'ticdesk_account'

    base_dn = 'ou=Groups,dc=teckids,dc=org'
    object_classes = ['posixGroup', 'groupOfNames', 'teckidsGroup']

    # Group name details    
    cn = CharField(db_column='cn', max_length=50, primary_key=True)
    display_name = CharField(db_column='displayName', max_length=100)
    description = CharField(db_column='description', max_length=200)

    # Members of this group
    members = ListField(db_column='member')
    member_uids = ListField(db_column='memberUid')
    owners = ListField(db_column='owner')

    def clean(self):
        self.sync_members()

    def sync_members(self):
        # Ensure member_uids are in sync mith members
        self.member_uids = []
        for member_dn in self.members + self.owners:
            # Rely on RDN being correctly set, for performance reasons
            rdn = member_dn.split(',')[0]
            rdn_parts = rdn.split('+')

            # Find the uid part of the RDN
            for rdn_part in rdn_parts:
                if rdn_part.startswith('uid='):
                    # Add to uids list and continue with next entry
                    uid = rdn_part.split('=')[1]
                    if uid not in self.member_uids:
                        self.member_uids.append(uid)
                    break

    def add_member(self, person):
        # Add person both to groupOfNames and posixGroup part
        if not person.dn in self.members:
            self.members.append(person.dn)
        if not person.uid in self.member_uids:
            self.member_uids.append(person.uid)

    def remove_member(self, person):
        # Remove person both from groupOfNames and posixGroup part
        if person.dn in self.members:
            self.members.remove(person.dn)
        if  person.uid in self.member_uids:
            self.member_uids.remove(person.uid)

    @property
    def members_persons(self):
        return self._get_objects('members', TeckidsPerson)

    @property
    def owners_persons(self):
        return self._get_objects('owners', TeckidsPerson)

    @property
    def group_type(self):
        if self.cn.startswith('team-'):
            return ('team', _('Allegemeine Teams'))
        elif self.cn.startswith('thema-'):
            return ('topic', _('Thementeams'))
        elif self.cn.startswith('news-'):
            return ('newsletter', _('Newsletter'))
        elif self.cn.startswith('project-'):
            return ('project', _('Projekte'))
        elif self.cn.startswith('projects-'):
            return ('project', _('Projekte'))
        elif self.cn in ['members', 'fritze', 'vorstand', 'vorstand-gf', 'themenwarte', 'paeda', 'unpaeda', 'aktive']:
            return ('commitee', _('Gremien'))
        else:
            return ('privilege', _('Privilegien'))

class TeckidsPerson(ldapdb.models.Model, TeckidsLdapMixin):
    class Meta:
        managed = False
        app_label = 'ticdesk_account'
        ordering = ('sn', 'given_name')

    base_dn = 'ou=People,dc=teckids,dc=org'
    object_classes = ['inetOrgPerson', 'organizationalPerson', 'teckidsPerson',
                      'posixAccount', 'shadowAccount', 'inetLocalMailRecipient']

    # Name
    cn = CharField(db_column='cn', max_length=200, primary_key=True, verbose_name=_('Name'))
    given_name = CharField(db_column='givenName', max_length=100, verbose_name=_('Vorname'))
    sn = CharField(db_column='sn', max_length=100, verbose_name=_('Nachname'))

    # Contact / identity details
    mail = CharField(db_column='mail', max_length=200, verbose_name=_('E-Mail'))
    home_postal_address = CharField(db_column='homePostalAddress', max_length=100, verbose_name=_('Anschrift (privat)'))
    home_phone = CharField(db_column='homePhone', max_length=20, verbose_name=_('Telefon (Festnetz privat)'))
    mobile = CharField(db_column='mobile', max_length=20, verbose_name=_('Handy'))
    date_of_birth = DateField(db_column='dateOfBirth', verbose_name=_('Geburtsdatum'))
    jpeg_photo = ImageField(db_column='jpegPhoto')

    # Location
    latitude = FloatField(db_column='teckidsLatitude')
    longitude = FloatField(db_column='teckidsLongitude')

    # Organisation (for children: school and class)
    o = CharField(db_column='o', max_length=50, verbose_name=_('Schule'))
    ou = CharField(db_column='ou', max_length=20, verbose_name=_('Klasse'))
    l = CharField(db_column='l', max_length=50, verbose_name=_('Schulort'))

    # Account details
    uid = CharField(db_column='uid', max_length=20, primary_key=True, verbose_name=_('Benutzername'))
    uid_number = IntegerField(db_column='uidNumber')
    gid_number = IntegerField(db_column='gidNumber', default=100)
    home_directory = CharField(db_column='homeDirectory', max_length=30)
    login_shell = CharField(db_column='loginShell', max_length=30, default='/bin/bash')
    user_password = CharField(db_column='userPassword', max_length=100)

    # Mail service details
    mail_addresses = ListField(db_column='mailLocalAddress')

    # Meta information (mostly used on members)
    title = CharField(db_column='title', max_length=100, verbose_name=_('Titel / Rolle'))
    description = CharField(db_column='description', max_length=5000, verbose_name=_('Beschreibung'))

    # Cash information
    #cash_account = IntegerField(db_column='teckidsCashAccount', verbose_name=_('Kontonummer'))
    iban = CharField(db_column='iBAN', max_length=50, verbose_name=_('IBAN'))
    bic = CharField(db_column='bIC', max_length=50, verbose_name=_('BIC'))

    # Member information
    employee_number = IntegerField(db_column='employeeNumber', verbose_name=_('Mitgliedsnummer'))
    member_since = DateField(db_column='teckidsMemberSince', verbose_name=_('Mitglied seit'))
    member_until = DateField(db_column='teckidsMemberUntil', verbose_name=_('Mitglied bis'))
    teckids_hour = CharField(db_column='teckidsHour', max_length=7, verbose_name=_('Teckids-Stunde'))

    # Group membership
    member_of = ListField(db_column='memberOf')

    # Settings
    setting_birthday_mail_days = IntegerField(db_column='teckidsSettingsBirthdayMailDays',
        default=0, verbose_name=_('Geburtstags-E-Mail-Tage')
    )

    def clean(self):
        # Set default cn if not given
        if not self.cn:
            self.cn = "%s %s" % (self.given_name, self.sn)

        # Set default home directory if not given
        if not self.home_directory:
            self.home_directory = "/home/%s" % self.uid

        # Choose next available uid number if not provided
        if not self.uid_number:
            self.uid_number = TeckidsPerson.objects.latest('uid_number').uid_number + 1

        # Activate SASL passthrough if principal exists
        if TeckidsKrbPrinc.objects.filter(princ_name='%s@TECKIDS.ORG' % self.uid):
            self.set_sasl_passthrough()

        # Update coordinates if postal address is given
        if self.homePostalAddress:
            location = nominatim.geocode(self.homePostalAddress)
            self.latitude, self.longitude = location.latitude, location.longitude

        # Remove from members group if membership ended
        if self.is_member_of('members') and self.member_until and self.member_until >= now:
            group = TeckidsGroup.objects.get(cn='members')
            group.remove_member(self)

        # Remove teckids.org addresses if not a member
        if not self.is_member:
            self.mail_addresses = [_ for _ in self.mail_addresses if not _.endswith('@teckids.org')]

        super(TeckidsPerson, self).clean()

    def save(self, *args, **kwargs):
        self.clean()
        super(TeckidsPerson, self).save(*args, **kwargs)

    def age_at(self, today):
        # Get years of age, taking leap years into account
        delta = today - self.date_of_birth
        years = int(delta.days / 365.2425)
        if self.date_of_birth > today - relativedelta(years=years):
            years -= 1
        return years

    @property
    def age(self):
        return self.age_at(date.today())

    def is_member_of(self, group_cn):
        return 'cn=%s,%s' % (group_cn, TeckidsGroup.base_dn) in self.member_of

    def is_owner_of(self, group_cn):
        return self in TeckidsGroup.objects.get(cn=group_cn).owners_persons

    @property
    def is_member(self):
        now = date.today()
        return self.is_member_of('members') and (self.member_since and self.member_since <= now) and (not self.member_until or self.member_until >= now)

    @property
    def is_board(self):
        return self.is_member and self.is_member_of('vorstand')

    @property
    def is_paeda(self):
        return self.is_member and self.is_member_of('paeda')

    @classmethod
    def create_person(cls, uid, gn, sn, dob, mail, password):
        # Create new LDAP object
        person = cls.objects.create(uid=uid, given_name=gn, sn=sn, date_of_birth=dob, mail=mail)
        if not person:
            return None

        # Set the password in LDAP
        person.user_password = ssha(password)
        person.save()

        # Do PAM authentication for the new user
        # On ticdesk.teckids.org, this will trigger pam_migrate to create the Kerberos principal
        p_conv = pam()
        res = p_conv.authenticate(uid, password, service='ticdesk')
        if not res:
            # PAM failed - remove the new user to not confuse systems!
            person.delete()
            return None
        # Save again, which will activate SASL passthrough in clean()
        person.save()

        return person

    def set_sasl_passthrough(self):
        # Activate SASL pass-through for LDAP simple binds
        # This finalises SSO for this person
        self.user_password = '{SASL}%s@TECKIDS.ORG' % self.uid

    def subscribe_newsletter(self, newsletter):
        # Add the person as group member of the desired newsletter LDAP group
        group = TeckidsGroup.objects.filter(
            cn='news-%s' % newsletter)[0]
        group.add_member(self)

    @property
    def groups_member(self):
        return self._get_objects('member_of', TeckidsGroup)

    @property
    def groups_owner(self):
        # We cannot use a memberOf overlay because we use nested groups, sadly…
        # FIXME This could probably be more efficient.
        return [group for group in TeckidsGroup.objects.all() if self in group.owners_persons]

    @property
    def jpeg_photo_url(self):
        return self.get_image_url('jpeg_photo')

    @property
    def home_postal_street(self):
        try:
            return self.home_postal_address.split(', ', 1)[0]
        except:
            return ''

    @property
    def home_postal_code(self):
        try:
            return self.home_postal_address.split(', ', 1)[1].split(' ', 1)[0]
        except:
            return ''

    @property
    def home_postal_place(self):
        try:
            return self.home_postal_address.split(', ', 1)[1].split(' ', 1)[1]
        except:
            return ''

    @property
    def teckids_hour_str(self):
        if self.teckids_hour:
            return time.strftime('%A %H:%M', time.strptime(self.teckids_hour, '%w %H:%M'))

class TeckidsKrbPrinc(ldapdb.models.Model):
    class Meta:
        managed = False
        app_label = 'ticdesk_account'

    base_dn = 'ou=KerberosPrincipals,dc=teckids,dc=org'
    object_classes = ['account', 'krb5Principal', 'krb5KDCEntry']

    # Only need the principal name as all we do is validating existence
    princ_name = CharField(db_column='krb5PrincipalName', max_length=100)
