import re

from django import forms
from django.utils.translation import ugettext_lazy as _
from localflavor.generic.forms import IBANFormField
from localflavor.generic.countries.sepa import IBAN_SEPA_COUNTRIES
import phonenumbers

from .models import Voucher

CHANNEL_CHOICES = [
    ('none', _('Keine Angabe')),
    ('internet', _('Aus dem Internet')),
    ('school', _('Aus der Schule')),
    ('friends', _('Von Freunden')),
    ('parents', _('Durch meine Eltern')),
    ('newsletter', _('Durch den Newsletter')),
]

SEX_CHOICES = [
    ('f', _('Mädchen (weiblich)')),
    ('m', _('Junge (männlich)')),
]

COMMENT_CHOICES = [
    ('first', _('Nur Vorname')),
    ('first_age', _('Vorname und Alter')),
    ('first_last_age', _('Vorname, Nachname und Alter')),
]

RATING_CHOICES = [('none', _('Keine Bewertung'))] + [
    (str(n), _('%d Stern(e)') % n) for n in range(1, 6)
]

LICENCE_CHOICES = [
    ('CC-BY-4.0+', 'Creative Commons mit Namensnennung, 4.0 oder später'),
    ('CC-BY-SA-4.0+', 'Creative Commons mit Namensnennung und Weitergabe nur unter gleichen Bedingungen, 4.0 oder später'),
]

def is_phonenumber(number):
    try:
        phonenumbers.parse(number, 'DE')
    except:
        raise forms.ValidationError(_('%s ist keine gültige Telefonnummer!') % number)

def is_valid_voucher_for(project_cn):
    def _is_valid_voucher(code):
        try:
            voucher = Voucher.objects.get(code=code, project_cn=project_cn, used=False)
        except Voucher.DoesNotExist:
            raise forms.ValidationError(_('Der Gutschein-Code ist ungültig.'))
    return _is_valid_voucher

def clean_phonenumber(field):
    def _clean_phonenumber(obj):
        value = obj.cleaned_data[field]

        if value:
            pn = phonenumbers.parse(value, 'DE')
            value = phonenumbers.format_number(pn, phonenumbers.PhoneNumberFormat.E164)

        return value
    return _clean_phonenumber

class EventRegisterForm(forms.Form):
    cn = forms.CharField(label=_('Dein Name'), disabled=True)
    date_of_birth = forms.DateField(label=_('Dein Geburtsdatum'), disabled=True)
    sex = forms.ChoiceField(label=_('Dein Geschlecht'), choices=SEX_CHOICES, widget=forms.RadioSelect,
        help_text=_('Aus verschiedenen Gründen, z.B. weil wir während der Übernachtungen aus rechtlichen Gründen Geschlechtertrennung einhalten müssen, müssen wir wissen, ob du ein Junge oder Mädchen bist. Bei manchen Namen ist das nicht immer sofort zu erkennen, daher bitten wir dich, es hier anzugeben.')
    )
    mail = forms.EmailField(label=_('Deine E-Mail-Adresse'), disabled=True)

    street = forms.CharField(label=_('Straße und Hausnummer'))
    postal_code = forms.CharField(label=_('Postleitzahl (PLZ)'))
    place = forms.CharField(label=_('Ort'))

    mobile = forms.CharField(required=False, label=_('Deine Handynummer'),
        help_text=_('Deine Handynummer hilft uns, dich im Notfall während der Veranstaltung zu erreichen, z.B. wenn du mit deiner Gruppe alleine auf einer Konferenz o.ä. unterwegs bist. Solltest du kein Handy haben, kannst du das Feld leer lassen.'),
        validators=[is_phonenumber]
    )

    school = forms.CharField(label=_('Schule'),
        help_text=_('Bitte gib den Namen deiner Schule möglichst genau so ein, wie er offiziell geschrieben wird.'))
    school_place = forms.CharField(label=_('Ort der Schule'),
        help_text=_('Gib hier den Ort (die Stadt) ein, in der deine Schule liegt (ohne Ortsteil).'))
    school_class = forms.CharField(label=_('Klasse'),
        help_text=_('Bitte gib die Klasse an, in die du gehst (z.B. 8a).'))

    # FIXME add full-featured guardian management
    guardian_name = forms.CharField(label=_('Vor- und Nachname eines Erziehungsberechtigten'),
        help_text=_('Bitte gib hier den Namen des Erziehungsberechtigten an, der die Anmeldung mit dir ausfüllt und im Notfall während der Veranstaltung erreichbar ist.')
    )
    guardian_mail = forms.EmailField(label=_('E-Mail-Adresse des Erziehungsberechtigten'))
    guardian_mobile = forms.CharField(label=_('Handynummer des Erziehungsberechtigten'),
        help_text=_('Die Handynummer benötigen wir für Notfälle, falls wir deine Eltern während der Veranstaltung dringend erreichen müssen.'),
        validators=[is_phonenumber]
    )

    channel = forms.ChoiceField(label=_('Wie hast du von der Veranstaltung erfahren?'), choices=CHANNEL_CHOICES)
    comment = forms.CharField(required=False, label=_('Kommentar / Bemerkungen'), widget=forms.Textarea)

    voucher_code = forms.CharField(required=False, label=_('Gutschein-Code'),
        help_text=_('Wenn du einen Gutschein für die Veranstaltung hast, gib hier den Code ein. Er wird automatisch verrechnet.')
    )

    donation = forms.IntegerField(required=False, min_value=0, label=_('Freiwilliger Mehrbetrag'),
        help_text=_('Unser Verein möchte allen Kindern und Jugendlichen die Möglichkeit bieten, an unseren Veranstaltungen teilzunehmen. Oft ist der Teilnehmerbeitrag für eine Familie jedoch nicht aufzubringen. Wir halten deshalb einen Etat bereit, aus dem wir Teilnahmen fördern können, nachdem wir die Notwendigkeit und Berechtigung sorgfältig geprüft haben. Für diesen Etat sind wir auf Spenden angewiesen. Wenn Sie einen freiwilligen Mehrbetrag für diesen Etat spenden möchten, geben Sie dies bitte hier an. Ob und wenn ja in welcher Höhe gespendet wurde, wird von uns nicht dauerhaft gespeichert und auch nicht innerhalb des Vereins, z.B. an Freizeitbetreuer, weitergegeben.')
    )

    accept_sepa = forms.BooleanField(required=False, label=_('Per SEPA-Lastschrift bezahlen'),
        help_text=_('Eltern: Ich ermächtige den Gläubiger Teckids e.V., Rochusstr. 2-4, 53123 Bonn mit der Gläubiger-ID DE70FZT00001497650, den Teilnehmerbeitrag von meinem Konto einmalig mit dem SEPA-Basislastschriftverfahren einzuziehen. Gleichzeitig weise ich meine Bank an, die vom Teckids e.V. von meinem Konto eingezogene SEPA-Basislastschrift einzulösen.')
    )

    iban = IBANFormField(required=False, label=_('IBAN (für SEPA-Lastschriftverfahren)'),
        help_text=_('Wenn deine Eltern per SEPA-Lastschrift bezahlen möchten, lasse sie bitte dieses Feld ausfüllen.'),
	include_countries=IBAN_SEPA_COUNTRIES
    )

    accept_terms = forms.BooleanField(label=_('Einverständniserklärung Eltern'),
        help_text=_('Eltern: Mein Kind hat das Anmeldeformular gemeinsam mit mir ausgefüllt und ich bin mit der Teilnahme einverstanden. Mir ist bewusst, dass die Anmeldung verbindlich ist und ein Rücktritt nur in Ausnahmefällen mit triftigem Grund möglich ist. Darüberhinaus erkläre ich mich einverstanden, den Teilnehmerbeitrag vorab zu begleichen und erkläre mich mit den oben genannten Erstattungsrichtlinien einverstanden.')
    )

    # Cleaning methods to reformat phone numbers
    clean_mobile = clean_phonenumber('mobile')
    clean_guardian_mobile = clean_phonenumber('guardian_mobile')

    def __init__(self, project, *args, **kwargs):
        super(EventRegisterForm, self).__init__(*args, **kwargs)
        self.fields['voucher_code'].validators.append(is_valid_voucher_for(project.cn))

class EventFeedbackForm(forms.Form):
    comment_private = forms.CharField(required=False, label=_('Kommentar für das Team'),
        help_text=_('Dieser Kommentar ist nur für das Team. Du kannst hier alles aufschreiben, was du uns noch als Feedback mitteilen möchtest.'),
        widget=forms.Textarea
    )

    comment_public = forms.CharField(required=False, label=_('Kommentar für die Website'),
        help_text=_('Dieser Kommentar ist für den Bericht auf unserer Website. Erzähle ausführlich davon, was du erlebt hast, was dir gefallen hat, was du gelernt hast und allem, was dir sonst noch einfällt.'),
        widget=forms.Textarea
    )

    comment_public_info = forms.ChoiceField(label=_('Angaben beim Kommentar?'), choices=COMMENT_CHOICES,
        help_text=_('Mit welchen Angaben möchtest du deinen Kommentar veröffentlichen?')
    )

    photos = forms.ImageField(label=_('Fotos'), widget=forms.ClearableFileInput(attrs={'multiple': True}),
        help_text=_('Wenn du Fotos für den Bericht beitragen möchtest, kannst du diese hier hochladen. Mehrere Dateien kannst du in den meisten Dateiauswahldialogen mit STRG+Klick auswählen.')
    )

    photos_licence = forms.ChoiceField(label=_('Foto-Lizenz'), choices=LICENCE_CHOICES,
        help_text=_('Wenn du Fotos hochlädst, wähle hier eine Lizenz aus.')
    )

    def __init__(self, project, *args, **kwargs):
        super(EventFeedbackForm, self).__init__(*args, **kwargs)

        for aspect in project.feedback_aspects:
            var = re.sub(r'\W|^(?=\d)', '_', aspect)
            field = forms.ChoiceField(label=_('Bewertung: %s') % aspect, choices=RATING_CHOICES,
                help_text=_('1 = nicht gut, 5 = sehr gut')
            )
            self.fields[var] = field
