from django.apps import apps
from django.http import JsonResponse, HttpResponse
from django.shortcuts import redirect, render
from django.template.loader import get_template
from django.urls import reverse
from django_tables2 import RequestConfig

from .tables import PersonTable
from .util import filter_disallowed_persons, get_relevant_groups, may_see_group, may_see_person, render_latex, tex_to_pdf

TeckidsPerson = apps.get_model('ticdesk_account', 'TeckidsPerson')
TeckidsGroup = apps.get_model('ticdesk_account', 'TeckidsGroup')

def person(request, uid=None):
    context = {}

    # Do not allow not logged-in users
    if request.user.is_anonymous():
        return HttpResponse(status=403)

    # Redirect to own listing if no uid given
    if uid is None:
        return redirect(reverse('org:person_by_uid', args=[request.user.username]))

    # Get person and check access    
    person = TeckidsPerson.objects.get(uid=uid)
    if not may_see_person(person, request.user):
        return HttpResponse(status=403)
    else:
        # Save person once to clean it, then add to context
        try:
            person.save()
        except:
            # Not being able to save here does not matter
            pass

        context['person'] = person

    # Add information for menu
    context['relevant_groups'] = get_relevant_groups(request.user)

    return render(request, 'ticdesk_org/person.html', context)

def persons(request, group=None):
    # Determine desired format
    format = request.GET.get('format', 'html')

    # For PDF format, determine template
    if format == 'pdf' or format == 'tex':
        template = request.GET.get('template', 'list_participants')

    context = {}

    if group:
        # Determine group pre-filter
        group = TeckidsGroup.objects.get(cn=group)

        if not may_see_group(group, request.user):
            return HttpResponse(status=403)
        else:
            persons = group.members_persons
            context['group'] = group

        # Evaluate QuerySet and filter for allowed persons
        persons = filter_disallowed_persons(list(persons), request.user)

        if format == 'html':
            # Build table
            person_table = PersonTable(persons)
            RequestConfig(request, paginate={'per_page': 25}).configure(person_table)
            context['person_table'] = person_table
        elif format == 'geojson':
            # Build GeoJSON dictionary
            geojson = []
            for person in persons:
                if person.latitude and person.longitude:
                    # Generate popup content from template
                    template = get_template('ticdesk_org/map_popup_content.html')
                    popup_content = template.render({'person': person})

                    # Add item to pre-GeoJSON structure
                    geojson.append({"type": "Feature",
                                    "properties": {
                                        "id": person.uid_number,
                                        "popupContent": popup_content
                                    },
                                    "geometry": {
                                        "type": "Point",
                                        "coordinates": [person.longitude, person.latitude]
                                    }})
        elif format == 'pdf' or format == 'tex':
            # Simply add persons to the context
            context['persons'] = persons

    if format == 'html':
        # Add information for menu
        context['relevant_groups'] = get_relevant_groups(request.user)
        return render(request, 'ticdesk_org/persons.html', context)
    elif format == 'geojson':
        return JsonResponse(geojson, safe=False)
    elif format == 'pdf' or format == 'tex':
        latex_str = render_latex('ticdesk_org/%s.tex' % template, context)

        if format == 'pdf':
            pdf = tex_to_pdf(latex_str)

            # Write HTTP response
            res = HttpResponse(pdf.data, content_type='application/pdf')
            res['Content-Disposition'] = 'attachment; filename="%s_%s.pdf"' % (template, group.cn)
            return res
        elif format == 'tex':
            # Write HTTP response
            res = HttpResponse(content_type='text/x-tex')
            res['Content-Disposition'] = 'attachment; filename="%s_%s.tex"' % (template, group.cn)
            res.write(latex_str)
            return res

def map(request):
    context = {}

    # Get group CN
    group_cn = request.GET.get('group_cn', '')

    # Load group
    group = TeckidsGroup.objects.get(cn=group_cn)
    if not may_see_group(group, request.user):
        return HttpResponse(status=403)

    # Pass title for heading in template
    context['title'] = group.display_name

    # Generate GeoJSON endpoint
    context['geojson'] = '%s?%s' % (reverse('org:persons_by_group', args=[group_cn]), 'format=geojson')

    # Add information for menu
    context['relevant_groups'] = get_relevant_groups(request.user)

    return render(request, 'ticdesk_org/map.html', context)
